#ifndef GAP_WORKSHEET_H
#define GAP_WORKSHEET_H

#include <moows/worksheet.h>
#include <moo/enum.h>
#include <moo/document.h>
#include <ggap/proc.h>

namespace ggap {

const char MimeWorksheet[] = "application/x-ggap-worksheet";
const char MimeWorkspace[] = "application/x-ggap-workspace";
const char MimePlainText[] = "text/plain";

class WorksheetScriptProxy;
class WorksheetPrivate;
class WorksheetDocument;
class Worksheet : public moo::ws::Worksheet {
    Q_OBJECT
    M_DECLARE_IMPL(Worksheet)

    friend class WorksheetDocument;
    friend class WorksheetScriptProxy;

public:
    Worksheet(QWidget *parent = 0);
    ~Worksheet();

    Q_INVOKABLE moo::Document *mdoc();

    enum ModeEnum {
        File,
        Terminal
    };
    M_DECLARE_ENUM(Mode, ModeEnum)

    Mode mode() const;
    void setMode(Mode mode);

    void start();
    bool isRunning();

    GapProcess::State gapState() const;

public Q_SLOTS:
    void interruptGap();
    void restartGap();
    void applyPrefs();

private:
    void processInput(const QStringList &lines);
    bool saveText(const QString &filename, QString &error);
    bool saveXml(const QString &filename, const QString &typeName, QString &error);
    bool loadText(const QString &filename, QString &error);
    bool loadWs(const QString &filename, QString &workspace, QString &error);

    bool loadFile(const QUrl &url, QString &error);
    bool saveFile(const QUrl &url, const QString &type, QString &error);

protected:
    void keyPressEvent(QKeyEvent *e);
    void resizeEvent(QResizeEvent *e);

private Q_SLOTS:
    void resize_idle();
    void gapScript(const QString &text);
    void gapMarkup(const QString &text);
    void gapOutput(const QString &text, GapProcess::OutputType type);
    void gapPrompt(const QString &text, bool first_time);
    void gapGlobalsChanged(const char *data, uint len, GapProcess::GlobalsChange);

    void gapStarted(GapProcess::StartResult result, const QString &output);
    void gapError(int exitCode, QProcess::ExitStatus exitStatus, const QString &output);
    void gapDied(int exitCode, QProcess::ExitStatus exitStatus);

Q_SIGNALS:
    void gapStateChanged();
    void gapExited();
};

} // namespace ggap

#endif // GAP_WORKSHEET_H
